class ChallengeForgeError(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return self.message


class InvalidContextError(ChallengeForgeError):
    pass


class OutputOverwriteError(ChallengeForgeError):
    pass


class QualityFailureError(ChallengeForgeError):
    pass
